package info.textgrid.middleware;

import java.util.Arrays;
import java.util.List;

/**
 * Collection of values never changing its content
 * 
 * @author Maximilian Brodhun: SUB
 *
 */

public final class TGConstants {

	private TGConstants() {
		// not called
	}

	// public static boolean furtherElements = true;

	public static final String			ITEM_IDENTIFIER_PREFIX		= "textgrid:";
	public static final List<String>	TEXTGRID_REP_ADMIN_CONTACT	= Arrays.asList("textgrid-support@gwdg.de");

	/*
	 * Rep Identification String Constants
	 */

	// public static final String TG_REP_NAME = "TextGrid Repository";
	public static final String			TG_REP_BASEURL				= "www.textgridrep.de";
	// public static final String TG_REP_OAIPMH_PROTOCOL_VERSION = "2.0";
	public static final String			METADATA_DC_PREFIX			= "oai_dc";

	/*
	 * Error String Constants
	 */

	public static final String			OAI_BAD_ARGUMENT			= "BadArgument";
	public static final String			OAI_METADATA_FORMAT_ERROR	= "FormatError";
	public static final String			OAI_NO_RECORD_MATCH			= "RecordMatchError";
	public static final String			OAI_NO_SET_HIERARCHY		= "SetHierarchyError";
	public static final String			OAI_BAD_RESUMPTION_TOKEN		= "badResumptionToken";

	/*
	 * TextGrid Metadata Fields for OAI-PMH Request (Mapping to DC)
	 */

	public static final String			CREATED						= "created";
	public static final String			FORMAT						= "format";
	public static final String			IDENTIFIER					= "identifier";
	public static final String			DATA_CONTRIBUTOR			= "dataContributor";
	public static final String			URI							= "textgridUri";
	public static final String			PROJECT_ID					= "project.id";
	public static final String			IS_DERIVED_FROM				= "relations.isDerivedFrom";
	public static final String			TITLE						= "title";
	public static final String			PID							= "pid.value";
	public static final String			EDITION_ISEDITIONOF			= "edition.isEditionOf";
	public static final String			EDITION_AGENT				= "edition.agent.value";
	public static final String			EDITION_LICENSEURI			= "edition.license.licenseUri";
	public static final String			EDITION_LANGUAGE			= "edition.language";
	public static final String			BIBCIT_AUTHOR				= "edition.source.bibliographicCitation.author.value";
	public static final String			BIBCIT_EDITOR				= "edition.source.bibliographicCitation.editor.value";
	public static final String			BIBCIT_TITLE				= "edition.source.bibliographicCitation.editionTitle";
	public static final String			BIBCIT_PLACEPUB				= "edition.source.bibliographicCitation.placeOfPublication.value";
	public static final String			BIBCIT_PUBLISHER			= "edition.source.bibliographicCitation.publisher.value";
	public static final String			BIBCIT_NO					= "edition.source.bibliographicCitation.editionNo";
	public static final String			BIBCIT_SERIES				= "edition.source.bibliographicCitation.series";
	public static final String			BIBCIT_VOLUME				= "edition.source.bibliographicCitation.volume";
	public static final String			BIBCIT_ISSUE				= "edition.source.bibliographicCitation.issue";
	public static final String			BIBCIT_EPAGE				= "edition.source.bibliographicCitation.epage";
	public static final String			BIBCIT_SPAGE				= "edition.source.bibliographicCitation.spage";
	public static final String			BIBCIT_BIBID				= "edition.source.bibliographicCitation.bibidentifier";
	public static final String			WORK_ABSTRACT				= "work.abstract";
	public static final String			WORK_GENRE					= "work.genre";
	public static final String			WORK_TYPE					= "work.type";
	public static final String			WORK_SPATIAL				= "work.spatial.value";
	public static final String			WORK_TEMPORAL				= "work.temporal.value";
	public static final String			WORK_AGENT					= "work.agent.value";
	public static final String			WORK_ID						= "work.subject.id.value";
	public static final String			MODIFIED_FIELD				= "lastModified";
	
	public static final String[]		TGFields					= {
			CREATED, FORMAT, IDENTIFIER, DATA_CONTRIBUTOR, URI, PROJECT_ID,
			IS_DERIVED_FROM, TITLE, PID, EDITION_ISEDITIONOF, EDITION_AGENT,
			EDITION_LICENSEURI, EDITION_LANGUAGE, BIBCIT_AUTHOR, BIBCIT_EDITOR,
			BIBCIT_TITLE, BIBCIT_PLACEPUB, BIBCIT_PUBLISHER, BIBCIT_NO,
			BIBCIT_SERIES, BIBCIT_VOLUME, BIBCIT_ISSUE, BIBCIT_EPAGE,
			BIBCIT_SPAGE, BIBCIT_BIBID, WORK_ABSTRACT, WORK_GENRE, WORK_TYPE,
			WORK_SPATIAL, WORK_TEMPORAL, WORK_AGENT	, MODIFIED_FIELD};

	public static final String[]		TGWorkFields				= {
			CREATED, WORK_ABSTRACT, IS_DERIVED_FROM, URI, WORK_GENRE, TITLE,
			WORK_TYPE, WORK_SPATIAL, WORK_TEMPORAL, WORK_AGENT, WORK_ID };

	/*
	 * String Arrays to define which TextGrid fields belongs to the regarding DC
	 * fields
	 */

	public static final String[]		CONTRIBUTOR_LIST			= { DATA_CONTRIBUTOR };
	public static final String[]		COVERAGE_LIST				= {
			WORK_SPATIAL, WORK_TEMPORAL							};
	public static final String[]		CREATOR_LIST				= { WORK_AGENT };
	public static final String[]		DATE_LIST					= { CREATED };
	public static final String[]		DESCRIPTION_LIST			= { WORK_ABSTRACT };
	public static final String[]		FORMAT_LIST				= { FORMAT };
	public static final String[]		IDENTIFIER_LIST			= { URI,PID, IDENTIFIER};
	public static final String[]		LANGUAGE_LIST				= { EDITION_LANGUAGE };
	public static final String[]		PUBLISHER_LIST				= { BIBCIT_PUBLISHER };
	public static final String[]		RELATIONS_LIST				= {
			PROJECT_ID, IS_DERIVED_FROM							};
	public static final String[]		RELATIONS_FOR_WORK_LIST		= {
			IS_DERIVED_FROM, TITLE, URI							};
	public static final String[]		RIGHTS_LIST					= { EDITION_LICENSEURI };
	public static final String[]		SOURCE_LIST					= {
			BIBCIT_AUTHOR, BIBCIT_EDITOR, BIBCIT_TITLE, BIBCIT_PLACEPUB,
			BIBCIT_PUBLISHER, BIBCIT_NO, BIBCIT_SERIES, BIBCIT_VOLUME,
			BIBCIT_ISSUE, BIBCIT_SPAGE, BIBCIT_EPAGE				};

	public static final String[]		SUBJECT_LIST				= { WORK_ID };
	public static final String[]		TITLE_LIST					= { TITLE };

	public static final String[]		TYPE_LIST					= {
			WORK_GENRE, WORK_TYPE									};

	public static final String[]		IDENTIFIER_LIST_FIELDS		= { URI,
			CREATED, FORMAT										};
	public static final String			RANGE_FIELD					= CREATED;
}
