package info.textgrid.middleware;

import static org.junit.Assert.assertTrue;
import java.io.IOException;
import java.net.URL;
import org.apache.cxf.helpers.IOUtils;
import org.apache.http.HttpStatus;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import info.textgrid.utils.httpclient.TGHttpClient;
import info.textgrid.utils.httpclient.TGHttpResponse;

/**
 * <p>
 * Some online tests for the DARIAH-DE Repository OAIMPH service.
 * </p>
 * 
 * @author Stefan E. Funk (SUB Göttingen)
 */

public class OaiPmhDariahdeOnlineTests {

  // The OAIPMH host to be tested.
  // private static String host =
  // "http://trep.de.dariah.eu/1.0/oaipmh/";
  private static String host = "https://repository.de.dariah.eu/1.0/oaipmh/";

  // Some output finals.
  private static final String ERROR = ">>> ERROR";
  private static final String OK = ">>> OKIDO";
  private static final String TESTING = "\n>>> TESTING ";

  // Time to wait between the single queries using resumption tokens in
  // milliseconds.
  private static final Long TIME = 30000l;

  // **
  // PREPARATIONS
  // **

  /**
   * @throws Exception
   */
  @BeforeClass
  public static void setUpBeforeClass() throws Exception {

  }

  /**
   * @throws Exception
   */
  @AfterClass
  public static void tearDownAfterClass() throws Exception {}

  /**
   * @throws Exception
   */
  @Before
  public void setUp() throws Exception {}

  /**
   * @throws Exception
   */
  @After
  public void tearDown() throws Exception {}

  // **
  // TESTS
  // **

  /**
   * @throws IOException
   */
  @Test
  public void testGetVersion() throws IOException {

    String shouldStartWith = "oaipmh-core";

    System.out.println(TESTING + "#GETVERSION for '" + shouldStartWith + "'");

    TGHttpResponse httpResponse = getHttpResponse(host + "oai/version");
    int status = httpResponse.getStatusCode();

    String response = IOUtils.readStringFromStream(httpResponse.getInputstream());

    if (status != HttpStatus.SC_OK || !response.startsWith(shouldStartWith)) {
      System.err.println("\tstatus: " + status);
      System.err.println(ERROR + ": response should start with '" + shouldStartWith + "'");
      assertTrue(false);
    }

    System.out.println("\tresponse: " + response);
    System.out.println(OK);
  }

  /**
   * @throws IOException
   */
  @Test
  public void testRootUrl() throws IOException {

    String shouldContain = "repository.de.dariah.eu";

    System.out.println(TESTING + "#ROOTURL");

    TGHttpResponse httpResponse = getHttpResponse(host + "oai");
    int status = httpResponse.getStatusCode();

    String response = IOUtils.readStringFromStream(httpResponse.getInputstream());

    if (status != HttpStatus.SC_OK || !response.contains(shouldContain)) {
      System.err.println("\tstatus: " + status);
      System.err.println(ERROR + ": response should contain '" + shouldContain + "'");
      assertTrue(false);
    }

    System.out.println("\tresponse: " + response);
    System.out.println(OK);
  }

  /**
   * @throws IOException
   */
  @Test
  public void testIdentify() throws IOException {

    String verb = "Identify";
    String shouldContain = "repository.de.dariah.eu";

    System.out.println(TESTING + "#IDENTIFY");

    TGHttpResponse httpResponse = getHttpResponse(host + "oai?verb=" + verb);
    int status = httpResponse.getStatusCode();

    String response = IOUtils.readStringFromStream(httpResponse.getInputstream());

    if (status != HttpStatus.SC_OK || !response.contains(shouldContain)) {
      System.err.println("\tstatus: " + status);
      System.err.println(ERROR + ": response should contain '" + shouldContain + "'");
      assertTrue(false);
    }

    System.out.println("\tresponse: " + response);
    System.out.println(OK);
  }

  /**
   * @throws IOException
   */
  @Test
  public void testListSets() throws IOException {

    String verb = "ListSets";
    // Testing for set: Bahnsen 1670_Erschließung_20161017.xlsx
    String shouldContain = "<setSpec>hdl:21.11113/0000-000B-CAD1-5</setSpec>";

    System.out.println(TESTING + "#LISTSETS");

    TGHttpResponse httpResponse = getHttpResponse(host + "oai?verb=" + verb);
    int status = httpResponse.getStatusCode();

    String response = IOUtils.readStringFromStream(httpResponse.getInputstream());

    if (status != HttpStatus.SC_OK || !response.contains(shouldContain)) {
      System.err.println("\tstatus: " + status);
      System.err.println(ERROR + ": response should contain '" + shouldContain + "'");
      assertTrue(false);
    }

    System.out.println("\tresponse: " + response);
    System.out.println(OK);
  }

  /**
   * @throws IOException
   */
  @Test
  public void testListRecordsSinglePage() throws IOException {

    System.out.println(TESTING + "#LISTRECORDS with expected single page (without token)");

    // Testing set: Bahnsen 1670_Erschließung_20161017.xlsx
    // String set = "21.11113/0000-000B-CAD1-5";
    String set = "";

    testList("ListRecords", set);
  }

  /**
   * @throws IOException
   */
  @Test
  public void testListRecordsMorePages() throws IOException {

    System.out.println(TESTING + "#LISTRECORDS with expected resumption token usage");

    // Testing set: Bahnsen 1670_Erschließung_20161017.xlsx
    // String set = "21.11113/0000-000B-CAD1-5";
    String set = "";

    testList("ListRecords", set);
  }

  /**
   * @throws IOException
   */
  @Test
  public void testListIdentifierSinglePage() throws IOException {

    System.out.println(TESTING + "#LISTIDENTIFIERS with expected single page (without token)");

    // Testing set: Bahnsen 1670_Erschließung_20161017.xlsx
    // String set = "21.11113/0000-000B-CAD1-5";
    String set = "";

    testList("ListIdentifiers", set);
  }

  /**
   * @throws IOException
   */
  @Test
  public void testListIdentifiersMorePages() throws IOException {

    System.out.println(TESTING + "#LISTIDENTIFIERS with expected resumption token usage");

    // Testing set: Bahnsen 1670_Erschließung_20161017.xlsx
    // String set = "21.11113/0000-000B-CAD1-5";
    String set = "";

    testList("ListIdentifiers", set);
  }

  // **
  // PRIVATE METHODS
  // **

  /**
   * @param theUrl
   * @return
   * @throws IOException
   */
  private static TGHttpResponse getHttpResponse(String theUrl) throws IOException {

    try {
      Thread.sleep(TIME);
    } catch (InterruptedException e) {
      // Nothing interrupts here!
    }

    System.out.println("\trequest:  " + theUrl);

    URL url = new URL(theUrl);
    String host = url.getProtocol() + "://" + url.getHost();
    String query = url.getPath() + "?" + url.getQuery();
    TGHttpClient client = new TGHttpClient(host);

    // Get response.
    TGHttpResponse response = client.get(query);

    // Check HTTP status.
    int status = response.getStatusCode();
    String reason = response.getReasonPhrase();

    if (status != HttpStatus.SC_OK) {
      System.err.println("\tstatus:   " + status + " " + reason);
      System.err.println(ERROR);
      assertTrue(false);
    } else {
      System.out.println("\tstatus:   " + status + " " + reason);
    }

    return response;
  }

  /**
   * @param theResponse
   * @return Resumption token string, if existing, -1 if either no <resumptionToken> tag is
   *         existing, tag is existing and has no token value.
   * @throws IOException
   */
  private static String examineResumptionTokenTag(TGHttpResponse theResponse, String recordOrHeader,
      String oldtok) throws IOException {

    String res = IOUtils.readStringFromStream(theResponse.getInputstream());

    // System.out.println(res);

    // Test for OAIPMH errors.
    if (res.contains("<error code=\"badArgument\">")) {
      System.err.println(ERROR + " IN OAIPMH RESPONSE: " + res);
      assertTrue(false);
    }

    // Count response objects at first.
    int recordCount = 0;
    int i = res.indexOf("<" + recordOrHeader + ">", 0);
    while (i != -1) {
      recordCount++;
      i++;
      i = res.indexOf("<" + recordOrHeader + ">", i);
    }

    System.out.println("\t" + recordOrHeader + "s:  " + recordCount);

    // Check if token tag is existing.
    int tokStart = res.indexOf("<resumptionToken");
    int tokEnd = res.indexOf("</resumptionToken");

    if (tokStart == -1 && tokEnd == -1) {
      System.out.println("\ttoken:    no token");

      return "-1";
    }

    String restokTmp = res.substring(tokStart, tokEnd);
    // Get token tag.
    String toktag = restokTmp.substring(0, restokTmp.indexOf(">") + 1).trim();
    System.out.println("\ttokentag: " + toktag);

    // Get token.
    String restok = restokTmp.substring(restokTmp.indexOf(">") + 1).trim();
    System.out.println("\ttoken:    " + restok);

    // Check if old and new token are equal or not.
    boolean tokchanged = !oldtok.equals(restok);
    System.out.println("\ttokchngd: " + tokchanged);

    // Get completeListSize and cursor.
    String sizeStr = toktag.substring(toktag.indexOf("completeListSize=\"") + 18);
    int size = Integer.parseInt(sizeStr.substring(0, sizeStr.indexOf("\"")));
    String cursorStr = toktag.substring(toktag.indexOf("cursor=\"") + 8);
    int cursor = Integer.parseInt(cursorStr.substring(0, cursorStr.indexOf("\"")));
    System.out.println("\tsize:     " + size + " / " + cursor);

    // If token is provided, and we have less than 100 elements: mekkern!
    if (!restok.isEmpty()) {
      synchronized (OaiPmhDariahdeOnlineTests.class) {
        // Check <record> or <header> count, must be 100!
        if (recordCount != 100) {
          System.err.println(ERROR + ": " + recordOrHeader
              + " count mismatch, must be 100 if token is provided, but is " + recordCount);
        }
        if (size <= 100) {
          System.err.println(ERROR
              + ": completeListSize count mismatch, must be > 100 if token is provided, but is "
              + size);
        }
        if (recordCount != 100 || size <= 100) {
          assertTrue(false);
        }
      }
    }

    // If no token is provided, stop querying.
    else {
      // Check <record> count, must be completeListSize % 100.
      if (recordCount != size % 100) {
        System.err.println(ERROR + ": " + recordOrHeader + " count mismatch, should be "
            + size % 100 + ", but is " + recordCount);
        assertTrue(false);
      }

      // No resumption token available in response.
      return "-1";
    }

    System.out.println("\t------------");

    return restok;
  }

  /**
   * @param theMillis
   * @return
   */
  public static String getDurationInSecs(long theMillis) {

    int SECS_IN_MILLIS = 1000;
    int secs = SECS_IN_MILLIS;

    return theMillis / secs + " second" + ((theMillis / secs) != 1 ? "s" : "");
  }

  /**
   * @param theVerb
   * @throws IOException
   */
  private static void testList(String theVerb, String theSet) throws IOException {

    long startTime = System.currentTimeMillis();

    String testOccurance = "header";
    if (theVerb.equals("ListRecords")) {
      testOccurance = "record";
    }

    String prefix = "oai_dc";

    String url = host + "oai?verb=" + theVerb + "&metadataPrefix=" + prefix;
    if (theSet != null && !theSet.equals("")) {
      url += "&set=" + theSet;
    }

    TGHttpResponse httpResponse = getHttpResponse(url);
    int status = httpResponse.getStatusCode();

    long timeRunning = System.currentTimeMillis() - startTime;
    System.out.println("\ttime:     " + getDurationInSecs(timeRunning));

    String restok = examineResumptionTokenTag(httpResponse, testOccurance, "");

    while (status == HttpStatus.SC_OK && !restok.equals("-1")) {
      url = host + "/oai?verb=" + theVerb + "&resumptionToken=" + restok;
      httpResponse = getHttpResponse(url);
      timeRunning = System.currentTimeMillis() - startTime;
      System.out.println("\ttime:     " + getDurationInSecs(timeRunning));
      restok = examineResumptionTokenTag(httpResponse, testOccurance, restok);
    }

    System.out.println(OK);
  }

}
